using System;
using System.IO;
using System.Text;
using System.Xml;

using Microsoft.BizTalk.Message.Interop;
using Microsoft.BizTalk.Component.Interop;

namespace Macaw.BizTalk.Pipeline
{
    /// <summary>
    /// The MessageFactory class contains helper methods for constructing pipeline
    /// message instances (from other messages).
    /// </summary>
    public static class MessageFactory
    {
        /// <summary>
        /// Creates a pipeline message from an xml document.
        /// </summary>
        /// <param name="msgDoc">The xml message document.</param>
        /// <param name="inMsg">The original pipeline message.</param>
        /// <param name="pipelineCtx">The pipeline context.</param>
        /// <returns>Returns a new pipeline message instance.</returns>
        /// <remarks>The method serializes the <paramref name="msgDoc"/> and creates
        /// a new pipeline message based on that stream.</remarks>
        public static IBaseMessage CreateMessage(XmlDocument msgDoc, IBaseMessage inMsg, IPipelineContext pipelineCtx)
        {
            // set data
            MemoryStream stream = new MemoryStream();

            XmlWriterSettings settings = new XmlWriterSettings();
            settings.Indent = false;
            settings.OmitXmlDeclaration = false;
            settings.Encoding = Encoding.UTF8;
            XmlWriter writer = XmlWriter.Create(stream, settings);

            msgDoc.Save(writer);
            stream.Position = 0;

            return CreateMessageFromStream(stream, inMsg, pipelineCtx);
        }

        /// <summary>
        /// Creates a pipeline message from a stream.
        /// </summary>
        /// <param name="msgStream">The message stream.</param>
        /// <param name="inMsg">The original pipeline message.</param>
        /// <param name="pipelineCtx">The pipeline context.</param>
        /// <returns>Returns a new pipeline message instance.</returns>
        /// <remarks>The method creates a new pipeline message based on <paramref name="msgStream"/>.
        /// The message context is copied onto the new message.</remarks>
        public static IBaseMessage CreateMessageFromStream(Stream msgStream, IBaseMessage inMsg, IPipelineContext pipelineCtx)
        {
            IBaseMessageFactory factory = pipelineCtx.GetMessageFactory();
            IBaseMessage msg = factory.CreateMessage();

            msg.Context = inMsg.Context;
            IBaseMessagePart bodyPart = factory.CreateMessagePart();

            // construct message body
            bodyPart.Charset = inMsg.BodyPart.Charset;
            bodyPart.ContentType = inMsg.BodyPart.ContentType;
            bodyPart.PartProperties = PipelineUtil.CopyPropertyBag(inMsg.BodyPart.PartProperties, factory);
            bodyPart.Data = msgStream;

            // also sets the body part
            CopyMessageParts(pipelineCtx, inMsg, msg, bodyPart, true);

            return msg;
        }

        // stolen from Microsoft.BizTalk.Pipeline.Components.dll
        /// <summary>
        /// copies all message parts part of the inbound message onto the outbound message
        /// </summary>
        /// <param name="pc">the <see cref="IPipelineContext"/> this message belongs to</param>
        /// <param name="inmsg">the inbound message</param>
        /// <param name="outmsg">the outbound message</param>
        /// <param name="bodyPart">the body part</param>
        /// <param name="allowUnrecognizeMessage">whether to allow unrecognized messages</param>
        internal static void CopyMessageParts(
            IPipelineContext pc,
            IBaseMessage inmsg,
            IBaseMessage outmsg,
            IBaseMessagePart bodyPart,
            bool allowUnrecognizeMessage)
        {
            string bodyPartName = inmsg.BodyPartName;
            for (int index = 0; index < inmsg.PartCount; index++)
            {
                string partName = null;

                IBaseMessagePart currentPart =
                    inmsg.GetPartByIndex(index, out partName);

                if ((currentPart == null) && !allowUnrecognizeMessage)
                {
                    throw new ArgumentNullException("otherOutPart[" + index + "]");
                }

                if (bodyPartName != partName)
                {
                    outmsg.AddPart(partName, currentPart, false);
                }
                else
                {
                    outmsg.AddPart(bodyPartName, bodyPart, true);
                }
            }
        }
    }
}
