using System;
using System.Collections;
using System.ComponentModel;
using System.Reflection;

using Microsoft.BizTalk.Component.Interop;
using Microsoft.BizTalk.Message.Interop;

namespace Macaw.BizTalk.Pipeline
{
    public abstract class PipelineComponentBase : 
        IBaseComponent, IPersistPropertyBag, 
        Microsoft.BizTalk.Component.Interop.IComponent
    {
        private bool _enabled = true;
        /// <summary>
        /// Gets or sets a value to enable (true) or disable (false) the
        /// pipeline component.
        /// </summary>
        [Browsable(true)]
        [Description("Enables (true) or disables (false) the pipeline component.")]
        public bool Enabled
        {
            get { return _enabled; }
            set { _enabled = value; }
        }

        protected virtual bool IsEnabled()
        {
            return Enabled;
        }

        #region IBaseComponent Members
        /// <summary>
        /// Description of the component
        /// </summary>
        [Browsable(false)]
        public abstract string Description {get;}
        /// <summary>
        /// Name of the component
        /// </summary>
        [Browsable(false)]
        public abstract string Name {get;}
        /// <summary>
        /// Version of the component
        /// </summary>
        [Browsable(false)]
        public virtual string Version
        {
            get { return Assembly.GetExecutingAssembly().GetName().Version.ToString(); }
        }

        #endregion

        #region IPersistPropertyBag Members

        public void GetClassID(out Guid classID)
        {
            classID = GetType().GUID;
        }

        public virtual void InitNew()
        {
            Enabled = true;
        }

        public virtual void Load(IPropertyBag propertyBag, int errorLog)
        {
            Enabled = ReadProperty<bool>(propertyBag, "Enabled", true);
        }

        public virtual void Save(IPropertyBag propertyBag, bool clearDirty, bool saveAllProperties)
        {
            WriteProperty(propertyBag, "Enabled", Enabled);
        }

        /// <summary>
        /// Reads property value from property bag
        /// </summary>
        /// <param name="pb">Property bag</param>
        /// <param name="propName">Name of property</param>
        /// <returns>Value of the property</returns>
        protected object ReadProperty(IPropertyBag pb, string propName, object defaultValue)
        {
            object val = null;

            try
            {
                pb.Read(propName, out val, 0);
            }
            catch (ArgumentException)
            {
                return defaultValue;
            }

            return val;
        }

        protected T ReadProperty<T>(IPropertyBag pb, string propName, T defaultValue)
        {
            object value = ReadProperty(pb, propName, null);

            if (value != null)
            {
                return (T)Convert.ChangeType(value, typeof(T));
            }

            return defaultValue;
        }

        /// <summary>
        /// Writes property values into a property bag.
        /// </summary>
        /// <param name="pb">Property bag.</param>
        /// <param name="propName">Name of property.</param>
        /// <param name="val">Value of property.</param>
        protected void WriteProperty(IPropertyBag pb, string propName, object val)
        {
            if (pb != null)
            {
                pb.Write(propName, ref val);
            }
        }
        #endregion

        #region IComponent Members

        public IBaseMessage Execute(IPipelineContext pContext, IBaseMessage pInMsg)
        {
            if (IsEnabled())
            {
                return ExecuteInternal(pContext, pInMsg);
            }
            else
            {
                return pInMsg;
            }
        }

        /// <summary>
        /// Derived classes implement
        /// </summary>
        protected abstract IBaseMessage ExecuteInternal(IPipelineContext pContext, IBaseMessage pInMsg);

        #endregion
    }
}
