using System;
using System.Collections.Generic;
using System.ComponentModel;
using System.Diagnostics;
using System.IO;
using System.Reflection;
using System.Text;
using System.Xml;
using System.Xml.Serialization;

using Microsoft.BizTalk.Component.Interop;
using Microsoft.BizTalk.Message.Interop;

namespace Macaw.BizTalk.Pipeline
{
    /// <summary>
    /// The eBizSaver class saves all messages to the database.
    /// </summary>
    [ComponentCategory(CategoryTypes.CATID_PipelineComponent)]
    [ComponentCategory(CategoryTypes.CATID_Any)]
    [System.Runtime.InteropServices.Guid("0f7b0116-d6d7-11db-9705-005056c00008")]
    public class StaticMessageContext : PipelineComponentWithResourceBase
    {
        public StaticMessageContext()
            : base("StaticMessageContext", Properties.Resources.ResourceManager)
        { }

        #region IComponent Members

        /// <summary>
        /// Called by the pipeline when a message arrives. 
        /// Applies the Context Properties configured to the message.
        /// </summary>
        /// <param name="pContext">Pipeline context</param>
        /// <param name="pInMsg">Message</param>
        /// <returns>Returns the original message (<paramref name="pInMsg"/>).</returns>
        protected override IBaseMessage ExecuteInternal(IPipelineContext pContext, IBaseMessage pInMsg)
        {
            try
            {
                ApplyContextProperties(pInMsg.Context);
            }
            catch (Exception e)
            {
                // TODO: Log errors
            }

            return pInMsg;
        }

        #endregion

        #region IPersistPropertyBag implementation
        
        public override void Load(IPropertyBag propertyBag, int errorLog)
        {
            base.Load(propertyBag, errorLog);

            int count = base.ReadProperty<int>(
                propertyBag, StaticMessageContextBagNames.Count, 0);

            for (int index = 0; index < count; index++)
            {
                ContextProperty ctxProp = new ContextProperty();

                ctxProp.Key = ReadProperty<string>(propertyBag, 
                    StaticMessageContextBagNames.FormatKey(index), String.Empty);
                ctxProp.Namespace = ReadProperty<string>(propertyBag,
                    StaticMessageContextBagNames.FormatNamespace(index), String.Empty);
                ctxProp.Promote = ReadProperty<bool>(propertyBag,
                    StaticMessageContextBagNames.FormatPromote(index), true);
                ctxProp.Value = ReadProperty<string>(propertyBag,
                    StaticMessageContextBagNames.FormatValue(index), String.Empty);

                ContextProperties.Add(ctxProp);
            }
        }

        public override void Save(IPropertyBag propertyBag, bool clearDirty, bool saveAllProperties)
        {
            base.Save(propertyBag, clearDirty, saveAllProperties);

            int count = ContextProperties.Count;
            WriteProperty(propertyBag, StaticMessageContextBagNames.Count, count);

            for(int index = 0; index < count; index++)
            {
                ContextProperty ctxProp = ContextProperties[index];

                WriteProperty(propertyBag, 
                    StaticMessageContextBagNames.FormatKey(index), ctxProp.Key);
                WriteProperty(propertyBag,
                    StaticMessageContextBagNames.FormatNamespace(index), ctxProp.Namespace);
                WriteProperty(propertyBag,
                    StaticMessageContextBagNames.FormatPromote(index), ctxProp.Promote);
                WriteProperty(propertyBag,
                    StaticMessageContextBagNames.FormatValue(index), ctxProp.Value);
            }
        }
        
        #endregion

        private ContextPropertyCollection _properties = new ContextPropertyCollection();
        public ContextPropertyCollection ContextProperties
        {
            get { return _properties; }
        }

        private void ApplyContextProperties(IBaseMessageContext msgCtx)
        {
            if (msgCtx != null && ContextProperties != null)
            {
                foreach (ContextProperty ctxProp in ContextProperties)
                {
                    if (ctxProp.Promote)
                    {
                        msgCtx.Promote(ctxProp.Key, ctxProp.Namespace, ctxProp.Value);
                    }
                    else
                    {
                        msgCtx.Write(ctxProp.Key, ctxProp.Namespace, ctxProp.Value);
                    }
                }
            }
        }
    }
}
